--[[

$Revision: 67 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Initialization

BossNotesAbilities = LibStub("AceAddon-3.0"):NewAddon("BossNotesAbilities",
	"AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes")


----------------------------------------------------------------------
-- Constants

-- Database version
BOSS_NOTES_ABILITIES_DATABASE_VERSION = 1

-- Provider name
BOSS_NOTES_ABILITIES_PROVIDER = "ABILITIES"

-- Difficulties
BOSS_NOTES_ABILITIES_DIFFICULTY_NORMAL = 1
BOSS_NOTES_ABILITIES_DIFFICULTY_HEROIC = 2

-- Ace configuration options
local ACE_OPTS = {
	name = L["ABILITIES"],
	handler = BossNotesAbilities,
	guiHidden = true,
	type = "group",
	args = {
		learn = {
			name = L["LEARN"],
			desc = L["LEARN_DESC"],
			order = 1,
			type = "toggle",
			get = "IsLearning",
			set = "ToggleLearning"
		},
		clear = {
			name = L["CLEAR"],
			desc = L["CLEAR_DESC"],
			order = 2,
			type = "execute",
			func = "ClearLearned"
		}
	}
}
do
	order = 1
	for _, opt in pairs(BOSS_NOTES_ACE_OPTS.args) do
		if opt.order >= order then
			order = opt.order + 1
		end
	end
	ACE_OPTS.order = order
	BOSS_NOTES_ACE_OPTS.args.abilities = ACE_OPTS
end

-- DB defaults
local DB_DEFAULTS = {
	global = {
		npcs = { },
		instanceEncounters = { }
	}
}

	
----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function BossNotesAbilities:OnInitialize ()
	-- Database
	self.db = LibStub("AceDB-3.0"):New("BossNotesAbilitiesDB")
	self.db:RegisterDefaults(DB_DEFAULTS)
	self.db.global.databaseVersion = BOSS_NOTES_DATABASE_VERSION
	
	-- Prune learned that are now in the encounter database
	for _, instanceEncounter in pairs(self.db.global.instanceEncounters) do
		for npcId in pairs(instanceEncounter.npcIds) do
			if BossNotes:GetInstanceAndEncounter(npcId) then
				instanceEncounter.npcIds[npcId] = nil
			end
		end
	end
end

-- Handles the (re-)enabling of this add-on
function BossNotesAbilities:OnEnable ()
	-- Register events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	-- Register provider
	BossNotes:RegisterProvider(BOSS_NOTES_ABILITIES_PROVIDER, {
		OnSources = function (instance, encounter) return self:OnSources(instance, encounter) end
	})
end

-- Handles the disabling of this add-on
function BossNotesAbilities:OnDisable ()
	-- Unregister provider
	BossNotes:UnregisterProvider(BOSS_NOTES_ABILITIES_PROVIDER)
end


----------------------------------------------------------------------
-- Event handlers

-- Returns the abilities note
function BossNotesAbilities:OnSources (instance, encounter)
	-- Get NPC IDs
	local npcIds = (encounter and encounter.npcIds) or (instance and instance.npcIds)
	local key = BossNotes:GetContextKey(instance, encounter)
	local instanceEncounter = self.db.global.instanceEncounters[key]
	if instanceEncounter then
		-- Merge
		local mergedNpcIds = { }
		if npcIds then
			for _, npcId in ipairs(npcIds) do
				if not instanceEncounter.npcIds[npcId] then
					table.insert(mergedNpcIds, npcId)
				end
			end
		end
		for npcId in pairs(instanceEncounter.npcIds) do
			table.insert(mergedNpcIds, npcId)
		end
		npcIds = mergedNpcIds
	end
	if not npcIds then
		return
	end
	
	-- Build HTML
	local html = { }
	table.insert(html, "<html><body>")
	for _, npcId in ipairs(npcIds) do
		local npc = self.db.global.npcs[npcId]
		if npc then
			-- Header
			table.insert(html, "<h1>" .. npc.name .. "</h1>")
			
			-- Acquire and sort spells
			local spells = { }
			for _, spell in pairs(npc.spells) do
				table.insert(spells, spell)
			end
			table.sort(spells, function (a, b)
				if a.difficulty ~= b.difficulty then
					return a.difficulty < b.difficulty
				end
				return a.name < b.name
			end)
			
			-- Append spells
			for _, spell in ipairs(spells) do
				local suffix = ""
				if bit.band(spell.difficulty, BOSS_NOTES_ABILITIES_DIFFICULTY_NORMAL) ~= 0 then
					suffix = suffix .. L["NORMAL"]
				end
				if bit.band(spell.difficulty, BOSS_NOTES_ABILITIES_DIFFICULTY_HEROIC) ~= 0 then
					if string.len(suffix) > 0 then
						suffix = suffix .. ", "
					end
					suffix = suffix .. L["HEROIC"]
				end
				table.insert(html, string.format("<p>" .. L["SPELL"] .. "</p>",
						spell.spellId, spell.name,	suffix))
			end
			table.insert(html, "<br />")
		end
	end
	
	-- Placeholder
	if #html == 1 then
		table.insert(html, "<p>" .. L["ABILITIES_NOT_RECORDED"] .. "</p>")
	end
	
	-- Finish
	table.insert(html, "</body></html>")
	
	-- Return
	return {
		title = L["ABILITIES"],
		content = table.concat(html, "\n")
	}
end


----------------------------------------------------------------------
-- Events

-- Handles a combat log event
function BossNotesAbilities:COMBAT_LOG_EVENT_UNFILTERED (message, timestamp, event,
		sourceGuid, sourceName, sourceFlags,
		destGuid, destName, destFlags, ...)
	-- Check event
	if event ~= "SPELL_CAST_START" and event ~= "SPELL_CAST_SUCCESS" then
		return
	end

	-- Check for NPC
	if string.sub(sourceGuid, 5, 5) ~= "3" then
		return
	end
	if bit.band(sourceFlags, COMBATLOG_OBJECT_REACTION_MASK) ~= COMBATLOG_OBJECT_REACTION_HOSTILE then
		return
	end
	local npcId = tonumber("0x" .. string.sub(sourceGuid, 9, 12))
	local instance, encounter = BossNotes:GetInstanceAndEncounter(npcId)
	if not instance then
		if self.learningZone and self.learningZone == GetRealZoneText() then
			instance, encounter = BossNotes:GetSelection()
			local key = BossNotes:GetContextKey(instance, encounter)
			local instanceEncounter = self.db.global.instanceEncounters[key]
			if not instanceEncounter then
				instanceEncounter = {
					contextText = BossNotes:GetContextText(instance, encounter),
					npcIds = { }
				}
				self.db.global.instanceEncounters[key] = instanceEncounter
			end
			if not instanceEncounter.npcIds[npcId] then
				BossNotes:Print(string.format(L["LEARNING"],
						sourceName, npcId, instanceEncounter.contextText))
				instanceEncounter.npcIds[npcId] = sourceName
				BossNotes:InvalidateContextKey(key)
			end
		else
			return
		end
	end
	
	-- Track NPC
	local npc = self.db.global.npcs[npcId]
	if not npc then
		npc = {
			name = sourceName,
			spells = { }
		}
		self.db.global.npcs[npcId] = npc
	end
	
	-- Track spell
	local spellId = select(1, ...)
	local spellName = select(2, ...)
	local spell = npc.spells[spellId]
	if not spell then
		spell = {
			spellId = spellId,
			name = spellName,
			difficulty = 0
		}
		npc.spells[spellId] = spell
	end
	
	-- Track difficulty
	local difficulty = bit.lshift(1, (IsInInstance() and GetCurrentDungeonDifficulty() or 1) - 1)
	if bit.band(spell.difficulty, difficulty) == 0 then
		spell.difficulty = bit.bor(spell.difficulty, difficulty)
		
		-- Invalidate
		BossNotes:InvalidateContextKey(BossNotes:GetContextKey(instance, encounter))
	end
end


----------------------------------------------------------------------
-- Options

-- Returns whether the module is learning about NPCs
function BossNotesAbilities:IsLearning (info)
	return self.learningZone
end

-- Starts learning about NPCs
function BossNotesAbilities:ToggleLearning (info)
	if not self.learningZone then
		self.learningZone = GetRealZoneText()
		BossNotes:Print(string.format(L["START_LEARNING"], self.learningZone))
	else
		self.learningZone = nil
		BossNotes:Print(L["STOP_LEARNING"])
	end
end

-- Clears learned instance encounters
function BossNotesAbilities:ClearLearned ()
	-- Clear
	local instanceEncounters = self.db.global.instanceEncounters
	self.db.global.instanceEncounters = { }
	BossNotes:Print(L["CLEAR_LEARNED"])
	
	-- Invalidate
	for key in pairs(instanceEncounters) do
		BossNotes:InvalidateContextKey(key)
	end
end
